using System;
using System.Text;
using System.IO;
using System.Net;
using System.Collections.Generic;
using Terrasoft.Nui.ServiceModel.DataContract;
using Terrasoft.Core.Entities;
using System.Web.Script.Serialization;

namespace DataServiceSelectExample
{
    class Program
    {
        // Основной Uri приложения bpm'online. Необходимо заменить на пользовательский.
        private const string baseUri = @"http://localhost/7.8.0.1006_SalesEnterprise_Demo_RUS_Simuta";
        // Строка запроса к методу Login сервиса AuthService.svc.
        private const string authServiceUri = baseUri + @"/ServiceModel/AuthService.svc/Login";
        // Строка пути запроса SelectQuery.
        private const string selectQueryUri = baseUri + @"/0/DataService/json/SyncReply/SelectQuery";
        // Cookie аутентификации bpm'online.
        private static CookieContainer AuthCookie = new CookieContainer();

        // Метод выполняет аутентификацию пользователя.
        // Параметры:
        // userName - имя пользователя bpm'online,
        // userPassword - пароль пользователя bpm'online.
        private static bool TryLogin(string userName, string userPassword)
        {
            // Создание экземпляра запроса к сервису аутентификации.
            var authRequest = HttpWebRequest.Create(authServiceUri) as HttpWebRequest;
            // Определение метода HTTP-запроса.
            authRequest.Method = "POST";
            // Определение типа контента запроса.
            authRequest.ContentType = "application/json";
            // Включение использования cookie в запросе.
            authRequest.CookieContainer = AuthCookie;
            // Помещение в тело запроса учетной информации пользователя.
            using (var requestStream = authRequest.GetRequestStream())
            {
                using (var writer = new StreamWriter(requestStream))
                {
                    writer.Write(@"{
                        ""UserName"":""" + userName + @""",
                        ""UserPassword"":""" + userPassword + @"""
                    }");
                }
            }

            // Получение ответа от сервера. Если аутентификация проходит успешно, в свойство AuthCookie будут 
            // помещены cookie, которые могут быть использованы для последующих запросов.
            using (var response = (HttpWebResponse)authRequest.GetResponse())
            {
                if (AuthCookie.Count > 0)
                {
                    return true;
                }
            }
            return false;
        }

        // Главный метод приложения.
        static void Main(string[] args)
        {
            //Если выполнение аутентификации неудачно, приложение завершает работу.
            if (!TryLogin("Пользователь 1", "Пользователь 1"))
            {
                Console.WriteLine("Ошибка аутентификации!");
                return;
            }

            // Экземпляр класса запроса.
            var selectQuery = new SelectQuery()
            {
                // Название корневой схемы.
                RootSchemaName = "Contact",
                // Добавление колонок в запрос.
                Columns = new SelectQueryColumns()
                {
                    // Коллекция колонок.
                    Items = new Dictionary<string, SelectQueryColumn>()
                    {
                        //Колонка [ФИО].
                        {
                            // Ключ.
                            "Name",
                            // Значение.
                            new SelectQueryColumn()
                            {
                                // Выражение, задающее тип колонки.
                                Expression = new ColumnExpression()
                                {
                                    // Тип выражения — колонка схемы.
                                    ExpressionType = EntitySchemaQueryExpressionType.SchemaColumn,
                                    // Путь к колонке.
                                    ColumnPath = "Name"
                                }
                            }
                        },
                        // Колонка [Дата рождения].
                        {
                            "BirthDate",
                            new SelectQueryColumn()
                            {
                                Expression = new ColumnExpression()
                                {
                                    ExpressionType = EntitySchemaQueryExpressionType.SchemaColumn,
                                    ColumnPath = "BirthDate"
                                }
                            }
                        }
                    }
                }
            };
            // Фильтры запроса.
            var selectFilters = new Filters()
            {
                // Тип фильтра — группа.
                FilterType = Terrasoft.Nui.ServiceModel.DataContract.FilterType.FilterGroup,
                // Коллекция фильтров.
                Items = new Dictionary<string, Filter>
                {
                    
                    // Фильтрация по году рождения.
                    {
                        // Ключ.
                        "FilterYear",
                        // Значение.
                        new Filter
                        {
                            // Тип фильтра — фильтр сравнения.
                            FilterType = Terrasoft.Nui.ServiceModel.DataContract.FilterType.CompareFilter,
                            // Тип сравнения — равенство.
                            ComparisonType = FilterComparisonType.Equal,
                            // Выражение, подлежащее проверке.
                            LeftExpression = new BaseExpression()
                            {
                                // Тип выражения — колонка схемы.
                                ExpressionType = EntitySchemaQueryExpressionType.SchemaColumn,
                                // Путь к схеме.
                                ColumnPath = "BirthDate"
                            },
                            // Выражение, с которым сравнивается проверяемое значение.
                            RightExpression = new BaseExpression
                            {
                                // Тип выражения — функция.
                                ExpressionType = EntitySchemaQueryExpressionType.Function,
                                // Тип функции — макрос.
                                FunctionType = FunctionType.Macros,
                                // Тип макроса — год.
                                MacrosType = EntitySchemaQueryMacrosType.Year,
                                // Аргумент функции.
                                FunctionArgument = new BaseExpression
                                {
                                    // Тип выражения, определяющего аргумент — параметр.
                                    ExpressionType = EntitySchemaQueryExpressionType.Parameter,
                                    // Инициализация параметра.
                                    Parameter = new Parameter
                                    {
                                        // Тип параметра — целое число.
                                        DataValueType = DataValueType.Integer,
                                        // Значение параметра.
                                        Value = "1992"
                                    }
                                }
                            }
                        }
                    }
                }
            };
            // Добавление фильтров в запрос.
            selectQuery.Filters = selectFilters;
            // Сериализация экземпляра класса запроса на чтение в JSON-строку.
            var json = new JavaScriptSerializer().Serialize(selectQuery);

            // Преобразование строки JSON-объекта в массив байтов.
            byte[] jsonArray = Encoding.UTF8.GetBytes(json);
            // Создание экземпляра HTTP-запроса.
            var selectRequest = HttpWebRequest.Create(selectQueryUri) as HttpWebRequest;
            // Определение метода запроса.
            selectRequest.Method = "POST";
            // Определение типа содержимого запроса.
            selectRequest.ContentType = "application/json";
            // Добавление полученных ранее аутентификационных cookie в запрос на получение данных.
            selectRequest.CookieContainer = AuthCookie;
            // Set the ContentLength property of the WebRequest.
            selectRequest.ContentLength = jsonArray.Length;
            // Помещение в содержимое запроса JSON-объекта.
            using (var requestStream = selectRequest.GetRequestStream())
            {
                requestStream.Write(jsonArray, 0, jsonArray.Length);
            }
            // Выполнение HTTP запроса и получение ответа от сервера.
            using (var response = (HttpWebResponse)selectRequest.GetResponse())
            {
                // Вывод ответа в консоль.
                using (StreamReader reader = new StreamReader(response.GetResponseStream()))
                {
                    Console.WriteLine(reader.ReadToEnd());
                }
            }
            // Задержка выполнения приложения.
            Console.ReadKey();

        }
    }
}
